/*
    ES Knot
    Copyright 2005, 2006 Martin Storsjo

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Martin Storsjo
    martin@martin.st
*/

#ifdef USE_OES_2
#include <EGL/egl.h>
#include <GLES2/gl2.h>
#include "knot2.h"
#else
#include <GLES/gl.h>
#include "knot.h"
#endif
#include <coemain.h>
#include "knotappview.h"
#include <flogger.h>
#include <hal.h>
#include <eikenv.h>
#ifdef UIQ3
#include <knot.rsg>
#else
#include <aknutils.h>
#include <aknnotewrappers.h>
#endif
#include <stdlib.h>
#include <string.h>

void debug(const char *str) {
	RFs &fs = CEikonEnv::Static()->FsSession();
	RFile file;
	_LIT(KFileName, "c:\\logs\\debug\\knotdbg.txt");
	TInt err;
	if ((err = file.Open(fs, KFileName, EFileWrite | EFileStreamText)) != KErrNone) {
		if ((err = file.Create(fs, KFileName, EFileWrite | EFileStreamText)) != KErrNone) {
			return;
		}
	}
	if (err == KErrNone) {
		TInt pos = 0;
		file.Seek(ESeekEnd, pos);
		file.Write(TPtrC8((const TUint8*) str, strlen(str)));
		file.Write(_L8("\n"));
		file.Close();
	}
}

void* customLoad(const char* name) {
	TInt ordinal = 0;
	if (!strcmp(name, "glGetFixedv")) {
		ordinal = 0x92;
	} else if (!strcmp(name, "glGetFloatv")) {
		ordinal = 0x93;
	} else {
		return NULL;
	}
	RLibrary dll;
	TInt err = dll.Load(_L("libGLES_CM.dll"));
	if (err == KErrNone) {
		void* ptr = (void*) dll.Lookup(ordinal);
		dll.Close();
		return ptr;
	}
	return NULL;
}

#ifdef UIQ3
CKnotAppView* CKnotAppView::NewL(CQikAppUi& aAppUi) {
	CKnotAppView* self = CKnotAppView::NewLC(aAppUi);
	CleanupStack::Pop(self);
	return self;
}

CKnotAppView* CKnotAppView::NewLC(CQikAppUi& aAppUi) {
	CKnotAppView* self = new(ELeave) CKnotAppView(aAppUi);
	CleanupStack::PushL(self);
	self->ConstructL(EWindowed, EEGLWindow, EFalse);
	return self;
}

CKnotAppView::CKnotAppView(CQikAppUi& aAppUi) : CGLAppView(aAppUi) {
}

#include "knotapp.h"
const TUid KUidKnotView = { 1 };
TVwsViewId CKnotAppView::ViewId() const {
	return TVwsViewId(KUidKnotApp, KUidKnotView);
}

TInt CKnotAppView::ViewConfigurationId() const {
	return R_KNOT_UI_CONFIGURATIONS;
}

#else
CKnotAppView* CKnotAppView::NewL(const TRect& aRect) {
	CKnotAppView* self = CKnotAppView::NewLC(aRect);
	CleanupStack::Pop(self);
	return self;
}

CKnotAppView* CKnotAppView::NewLC(const TRect& aRect) {
	CKnotAppView* self = new(ELeave) CKnotAppView();
	CleanupStack::PushL(self);

	TWindowMode mode = EFullscreen;
#ifdef EKA2
	// On S60v5 we may not have hardware softkeys, and thus
	// can't escape the fullscreen mode. Start in windowed mode
	// if pen support is enabled.
#if !defined(__WINSCW__)
	RLibrary dll;
	TInt err = dll.Load(_L("avkon.dll"), TUidType());
	if (err == KErrNone) {
		typedef TBool (*PenEnabledPtr)();
		PenEnabledPtr PenEnabled;
		PenEnabled = (PenEnabledPtr) dll.Lookup(0x109b);
		if (PenEnabled && PenEnabled())
			mode = EWindowed;
		dll.Close();
	}
#endif
#endif
	self->ConstructL(aRect, mode, EEGLWindow, EFalse);
	return self;
}

CKnotAppView::CKnotAppView() {
}

#endif

void CKnotAppView::DeinitGC() {
	if (iFont && iGc)
		iGc->DiscardFont();
}

void CKnotAppView::InitGC() {
#ifdef UIQ3
	iFont = &ScreenFont(TCoeFont::NormalFont());
#else
//	iFont = LatinBold13();
	iFont = LatinPlain12();
//	iFont = NumberPlain5();
#endif
	iGc->UseFont(iFont);
}

CKnotAppView::~CKnotAppView() {
	DeinitGC();
	delete iKnot;

	CloseSTDLIB();
}

void CKnotAppView::ConstructAppL() {
	iKnot = new(ELeave) Knot(Rect().Width(), Rect().Height());
	iKnot->setUpdateVertices(true);
	iKnot->setTexMode(Knot::TEXSPEC);
	EnableDragEvents();
}

void CKnotAppView::ReinitApp() {
	if (iKnot)
		iKnot->reinit();
}

void CKnotAppView::AppClearContext() {
	if (iKnot)
		iKnot->clearContext();
}

void CKnotAppView::SizeChanged() {
	if (iKnot)
		iKnot->setSize(Rect().Width(), Rect().Height());

	CGLAppView::SizeChanged();
}

void CKnotAppView::UpdateAndDraw() {
	iKnot->draw(iTime);

	FinishGLDrawing();

	if (iDrawMode != EEGLWindow) {
		eglWaitGL();
		TPoint point(2, 2 + iFont->HeightInPixels());
		TBuf<20> buf;
		buf.Format(_L("%d fps"), iLastFPS);
#ifdef EKA2
		iGc->SetPenColor(TRgb(255, 255, 255, 0)); // zero alpha, don't XOR that one
#else
		iGc->SetPenColor(KRgbWhite);
#endif
		iGc->SetDrawMode(CGraphicsContext::EDrawModeXOR);
		iGc->DrawText(buf, point);
		TInt ticklength = iRelTick - iFrameTick[iFrame % iFrameTickLength];
		buf.Format(_L("%d ticks/%d frames"), ticklength, iFrameTickLength);
		point.iX = Rect().Width() - 2 - iFont->TextWidthInPixels(buf);
		iGc->DrawText(buf, point);
		eglWaitNative(EGL_CORE_NATIVE_ENGINE);
	}
}

void CKnotAppView::FPSUpdated() {
	RFileLogger::WriteFormat(_L("debug"), _L("knot.txt"), EFileLoggingModeAppend, _L("%d fps"), iLastFPS);
}

TKeyResponse CKnotAppView::OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType) {
	if (aType != EEventKeyDown)
		return EKeyWasNotConsumed;

	if (aKeyEvent.iScanCode == EStdKeyDevice3) {
		if (iKnot)
			iKnot->startMorph();
		return EKeyWasConsumed;
	}
	return EKeyWasNotConsumed;
}

void CKnotAppView::HandlePointerEventL(const TPointerEvent& aPointerEvent) {
	switch (aPointerEvent.iType) {
	case TPointerEvent::EButton1Down:
		iLastPos = aPointerEvent.iPosition;
		iKnot->enableRotate(false);
		iDistance = 0;
		break;
	case TPointerEvent::EButton1Up:
		iKnot->enableRotate(true);
		if (iKnot && iDistance < 20)
			iKnot->startMorph();
		break;
	case TPointerEvent::EDrag: {
		TPoint diff = aPointerEvent.iPosition - iLastPos;
		iDistance += Abs(diff.iX) + Abs(diff.iY);
		iKnot->rotate(diff.iX, diff.iY);
		iLastPos = aPointerEvent.iPosition;
	}
		break;
	default:
		break;
	}
}

